@@ -1,3 +1,5 @@
+from importlib import resources
+
 from PyQt5.QtWidgets import (
     QVBoxLayout,
     QHBoxLayout,
@@ -10,13 +12,16 @@ from PyQt5.QtWidgets import (
     QCheckBox,
     QStyledItemDelegate,
     QLabel,
-    QListWidget,
+    QSizePolicy,
 )
+from PyQt5.QtGui import QIcon, QFont
 from PyQt5.QtCore import QRegExp, QPropertyAnimation
 
 from badger.gui.default.components.collapsible_box import CollapsibleBox
 from badger.gui.default.components.var_table import VariableTable
 from badger.gui.default.components.obj_table import ObjectiveTable
+from badger.gui.default.components.con_table import ConstraintTable
+from badger.gui.default.components.obs_table import ObservableTable
 from badger.gui.default.components.data_table import init_data_table
 from badger.settings import init_settings
 from badger.gui.default.utils import (
@@ -60,6 +65,22 @@ stylesheet_manual_msg = """
     }
 """
 
+stylesheet_load = """
+QPushButton:hover:pressed
+{
+    background-color: #4DB6AC;
+}
+QPushButton:hover
+{
+    background-color: #26A69A;
+}
+QPushButton
+{
+    color: #FFFFFF;
+    background-color: #00897B;
+}
+"""
+
 MSG_AUTO_RELATIVE = (
     "The values you see in the variable ranges spin "
     "boxes and initial points table are preview that "
@@ -95,9 +116,34 @@ class BadgerEnvBox(QWidget):
 
     def init_ui(self):
         config_singleton = init_settings()
+
+        icon_ref = resources.files(__package__) / "../images/import.png"
+        with resources.as_file(icon_ref) as icon_path:
+            self.icon_import = QIcon(str(icon_path))
+
         vbox = QVBoxLayout(self)
         vbox.setContentsMargins(8, 8, 8, 8)
 
+        # Load Template Button
+        template_button = QWidget()
+        # template_button.setFixedWidth(128)
+        hbox_temp = QHBoxLayout(template_button)
+        hbox_temp.setContentsMargins(0, 0, 0, 0)
+
+        cool_font = QFont()
+        cool_font.setWeight(QFont.DemiBold)
+
+        self.load_template_button = load_template_button = QPushButton(" Load Template")
+        load_template_button.setFixedHeight(36)
+        load_template_button.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
+        load_template_button.setIcon(self.icon_import)
+        load_template_button.setFont(cool_font)
+        load_template_button.setStyleSheet(stylesheet_load)
+
+        hbox_temp.addWidget(load_template_button, 1)
+        vbox.addWidget(template_button)
+        template_button.show()
+
         self.setObjectName("EnvBox")
 
         name = QWidget()
@@ -114,6 +160,8 @@ class BadgerEnvBox(QWidget):
         btn_env_play.setFixedSize(128, 24)
         if not strtobool(config_singleton.read_value("BADGER_ENABLE_ADVANCED")):
             btn_env_play.hide()
+        self.btn_pv = btn_pv = QPushButton("Variable Search")
+        btn_pv.setFixedSize(128, 24)
         self.btn_docs = btn_docs = QPushButton("Open Docs")
         btn_docs.setFixedSize(96, 24)
         self.btn_params = btn_params = QPushButton("Parameters")
@@ -125,6 +173,7 @@ class BadgerEnvBox(QWidget):
         hbox_name.addWidget(cb, 1)
         hbox_name.addWidget(btn_env_play)
         hbox_name.addWidget(btn_params)
+        hbox_name.addWidget(btn_pv)
         hbox_name.addWidget(btn_docs)
         vbox.addWidget(name)
 
@@ -361,21 +410,23 @@ class BadgerEnvBox(QWidget):
         edit_con_col = QWidget()
         vbox_con_edit = QVBoxLayout(edit_con_col)
         vbox_con_edit.setContentsMargins(0, 0, 0, 0)
+
         action_con = QWidget()
         hbox_action_con = QHBoxLayout(action_con)
         hbox_action_con.setContentsMargins(0, 0, 0, 0)
         vbox_con_edit.addWidget(action_con)
-        self.btn_add_con = btn_add_con = QPushButton("Add")
-        btn_add_con.setFixedSize(96, 24)
-        btn_add_con.setDisabled(True)
-        hbox_action_con.addWidget(btn_add_con)
+        self.edit_con = edit_con = QLineEdit()
+        edit_con.setPlaceholderText("Filter constraints...")
+        edit_con.setFixedWidth(192)
+        self.check_only_con = check_only_con = QCheckBox("Show Checked Only")
+        check_only_con.setChecked(False)
+        hbox_action_con.addWidget(edit_con)
         hbox_action_con.addStretch()
-        self.list_con = QListWidget()
-        self.list_con.setMaximumHeight(80)
-        # self.list_con.setFixedHeight(64)
-        self.list_con.setViewportMargins(2, 2, 17, 2)
-        vbox_con_edit.addWidget(self.list_con)
-        # vbox_con_edit.addStretch()
+        hbox_action_con.addWidget(check_only_con)
+
+        self.con_table = ConstraintTable()
+        self.con_table.setMinimumHeight(120)
+        vbox_con_edit.addWidget(self.con_table)
         hbox_con.addWidget(edit_con_col)
 
         # States config
@@ -395,22 +446,25 @@ class BadgerEnvBox(QWidget):
         edit_sta_col = QWidget()
         vbox_sta_edit = QVBoxLayout(edit_sta_col)
         vbox_sta_edit.setContentsMargins(0, 0, 0, 0)
+
         action_sta = QWidget()
         hbox_action_sta = QHBoxLayout(action_sta)
         hbox_action_sta.setContentsMargins(0, 0, 0, 0)
         vbox_sta_edit.addWidget(action_sta)
-        self.btn_add_sta = btn_add_sta = QPushButton("Add")
-        btn_add_sta.setFixedSize(96, 24)
-        btn_add_sta.setDisabled(True)
-        hbox_action_sta.addWidget(btn_add_sta)
+        self.edit_sta = edit_sta = QLineEdit()
+        edit_sta.setPlaceholderText("Filter observables...")
+        edit_sta.setFixedWidth(192)
+        self.check_only_sta = check_only_sta = QCheckBox("Show Checked Only")
+        check_only_sta.setChecked(False)
+        hbox_action_sta.addWidget(edit_sta)
         hbox_action_sta.addStretch()
-        self.list_obs = QListWidget()
-        self.list_obs.setMaximumHeight(80)
-        # self.list_obs.setFixedHeight(64)
-        self.list_obs.setViewportMargins(2, 2, 17, 2)
-        vbox_sta_edit.addWidget(self.list_obs)
-        # vbox_sta_edit.addStretch()
+        hbox_action_sta.addWidget(check_only_sta)
+
+        self.sta_table = ObservableTable()
+        self.sta_table.setMinimumHeight(120)
+        vbox_sta_edit.addWidget(self.sta_table)
         hbox_sta.addWidget(edit_sta_col)
+
         cbox_more.setContentLayout(vbox_more)
 
     def config_logic(self):
@@ -420,6 +474,10 @@ class BadgerEnvBox(QWidget):
         self.check_only_var.stateChanged.connect(self.toggle_var_show_mode)
         self.edit_obj.textChanged.connect(self.filter_obj)
         self.check_only_obj.stateChanged.connect(self.toggle_obj_show_mode)
+        self.edit_con.textChanged.connect(self.filter_con)
+        self.check_only_con.stateChanged.connect(self.toggle_con_show_mode)
+        self.edit_sta.textChanged.connect(self.filter_sta)
+        self.check_only_sta.stateChanged.connect(self.toggle_sta_show_mode)
         self.btn_params.toggled.connect(self.toggle_params)
         self.animation.finished.connect(self.animation_finished)
 
@@ -460,19 +518,22 @@ class BadgerEnvBox(QWidget):
         self.var_table.update_variables(_variables, 1)
 
     def toggle_obj_show_mode(self, _):
-        self.obj_table.toggle_show_mode(self.check_only_obj.isChecked())
+        self.obj_table.update_show_selected_only(self.check_only_obj.isChecked())
 
     def filter_obj(self):
-        keyword = self.edit_obj.text()
-        rx = QRegExp(keyword)
+        self.obj_table.update_keyword(self.edit_obj.text())
 
-        _objectives = []
-        for obj in self.obj_table.all_objectives:
-            oname = next(iter(obj))
-            if rx.indexIn(oname, 0) != -1:
-                _objectives.append(obj)
+    def toggle_con_show_mode(self, _):
+        self.con_table.update_show_selected_only(self.check_only_con.isChecked())
 
-        self.obj_table.update_objectives(_objectives, 1)
+    def filter_con(self):
+        self.con_table.update_keyword(self.edit_con.text())
+
+    def toggle_sta_show_mode(self, _):
+        self.sta_table.update_show_selected_only(self.check_only_sta.isChecked())
+
+    def filter_sta(self):
+        self.sta_table.update_keyword(self.edit_sta.text())
 
     def _fit_content(self, list):
         height = list.sizeHintForRow(0) * list.count() + 2 * list.frameWidth() + 4
@@ -481,9 +542,6 @@ class BadgerEnvBox(QWidget):
 
     def fit_content(self):
         return
-        self._fit_content(self.list_obj)
-        self._fit_content(self.list_con)
-        self._fit_content(self.list_obs)
 
     def switch_var_panel_style(self, auto=True):
         if auto:
@@ -501,7 +559,7 @@ class BadgerEnvBox(QWidget):
             stylesheet = f"""
                 #EnvBox {{
                     border-radius: 4px;
-                    border-color: {color_dict['normal']};
+                    border-color: {color_dict["normal"]};
                     border-width: 4px;
                 }}
             """
