@@ -21,7 +21,6 @@ from xopt.generators import get_generator_defaults, all_generator_names
 from xopt.utils import get_local_region
 
 from badger.gui.default.components.generator_cbox import BadgerAlgoBox
-from badger.gui.default.components.constraint_item import constraint_item
 from badger.gui.default.components.data_table import (
     get_table_content_as_dict,
     set_init_data_table,
@@ -53,12 +52,6 @@ from badger.utils import (
     get_xopt_version,
 )
 
-CONS_RELATION_DICT = {
-    ">": "GREATER_THAN",
-    "<": "LESS_THAN",
-    "=": "EQUAL_TO",
-}
-
 
 class BadgerRoutinePage(QWidget):
     sig_updated = pyqtSignal(str, str)  # routine name, routine description
@@ -217,7 +210,7 @@ class BadgerRoutinePage(QWidget):
         self.generators = list_generators()
         self.envs = list_env()
         # Clean up the constraints/observables list
-        self.env_box.list_con.clear()
+        self.env_box.con_table.clear_constraints()
         self.env_box.list_obs.clear()
 
         if routine is None:
@@ -519,7 +512,7 @@ class BadgerRoutinePage(QWidget):
         self.env_box.check_only_obj.setChecked(False)
         self.env_box.obj_table.update_objectives(objs_env)
 
-        self.env_box.list_con.clear()
+        self.env_box.con_table.clear_constraints()
         self.env_box.list_obs.clear()
         self.env_box.fit_content()
         # self.routine = None
@@ -544,7 +537,7 @@ class BadgerRoutinePage(QWidget):
         env = self.create_env()
         table = self.env_box.init_table
         vname_selected = self.get_init_table_header()
-        var_curr = env._get_variables(vname_selected)
+        var_curr = env.get_variables(vname_selected)
 
         # Iterate through the rows
         for row in range(table.rowCount()):
@@ -554,7 +547,7 @@ class BadgerRoutinePage(QWidget):
             ):
                 # Fill the row with content_list
                 for col, name in enumerate(vname_selected):
-                    item = QTableWidgetItem(f"{var_curr[name]:.4g}")
+                    item = QTableWidgetItem(f"{var_curr[name]:.6g}")
                     table.setItem(row, col, item)
                 break  # Stop after filling the first non-empty row
 
@@ -571,7 +564,7 @@ class BadgerRoutinePage(QWidget):
         # Get current point
         env = self.create_env()
         vname_selected = self.get_init_table_header()
-        var_curr = env._get_variables(vname_selected)
+        var_curr = env.get_variables(vname_selected)
 
         # get small region around current point to sample
         vocs, _ = self._compose_vocs()
@@ -601,7 +594,7 @@ class BadgerRoutinePage(QWidget):
                 try:
                     point = random_points.pop(0)
                     for col, name in enumerate(vname_selected):
-                        item = QTableWidgetItem(f"{point[name]:.4g}")
+                        item = QTableWidgetItem(f"{point[name]:.6g}")
                         table.setItem(row, col, item)
                 except IndexError:  # No more points to add
                     break
@@ -689,7 +682,7 @@ class BadgerRoutinePage(QWidget):
                 vrange[name] = var[name]
 
         env = self.create_env()
-        var_curr = env._get_variables(vname_selected)
+        var_curr = env.get_variables(vname_selected)
 
         option_idx = self.limit_option["limit_option_idx"]
         if option_idx:
@@ -767,7 +760,7 @@ class BadgerRoutinePage(QWidget):
             vrange[name] = var[name]
 
         env = self.create_env()
-        var_curr = env._get_variables(vname_selected)
+        var_curr = env.get_variables(vname_selected)
 
         for name in vname_selected:
             try:
@@ -831,20 +824,13 @@ class BadgerRoutinePage(QWidget):
             return
 
         options = self.configs["observations"]
-        item = QListWidgetItem(self.env_box.list_con)
-        con_item = constraint_item(
+        self.env_box.con_table.add_constraint(
             options,
-            lambda: self.env_box.list_con.takeItem(self.env_box.list_con.row(item)),
             name,
             relation,
             threshold,
             critical,
         )
-        item.setSizeHint(con_item.sizeHint())
-        self.env_box.list_con.addItem(item)
-        self.env_box.list_con.setItemWidget(item, con_item)
-        # self.env_box.dict_con[''] = item
-        self.env_box.fit_content()
 
     def add_state(self, name=None):
         if self.configs is None:
@@ -870,14 +856,14 @@ class BadgerRoutinePage(QWidget):
 
         constraints = {}
         critical_constraints = []
-        for i in range(self.env_box.list_con.count()):
-            item = self.env_box.list_con.item(i)
-            item_widget = self.env_box.list_con.itemWidget(item)
-            critical = item_widget.check_crit.isChecked()
-            con_name = item_widget.cb_obs.currentText()
-            relation = CONS_RELATION_DICT[item_widget.cb_rel.currentText()]
-            value = item_widget.sb.value()
-            constraints[con_name] = [relation, value]
+        for (
+            con_name,
+            relation,
+            threshold,
+            critical,
+            _,
+        ) in self.env_box.con_table.export_constraints():
+            constraints[con_name] = [relation, threshold]
             if critical:
                 critical_constraints.append(con_name)
 
