@@ -3,10 +3,13 @@ import traceback
 from importlib import resources
 from typing import List
 
+from badger.gui.default.components.analysis_extensions import AnalysisExtension
 import numpy as np
 import pandas as pd
 import pyqtgraph as pg
 from PyQt5.QtCore import pyqtSignal
+from pyqtgraph.Qt import QtGui, QtCore
+
 from PyQt5.QtGui import QIcon
 from PyQt5.QtWidgets import (
     QCheckBox,
@@ -34,6 +37,10 @@ from badger.gui.default.windows.terminition_condition_dialog import (
 from badger.gui.default.components.extensions_palette import ExtensionsPalette
 from badger.gui.default.components.routine_runner import BadgerRoutineSubprocess
 
+import logging
+
+logger = logging.getLogger(__name__)
+
 # disable chained assignment warning from pydantic
 pd.options.mode.chained_assignment = None  # default='warn'
 
@@ -85,7 +92,7 @@ class BadgerOptMonitor(QWidget):
         self.termination_condition = None
 
         self.extensions_palette = ExtensionsPalette(self)
-        self.active_extensions = []
+        self.active_extensions: list[AnalysisExtension] = []
 
         self.testing = False
         self.tc_dialog = None
@@ -165,7 +172,10 @@ class BadgerOptMonitor(QWidget):
         monitor.nextRow()
 
         self.plot_var = plot_var = add_axes(
-            monitor, "variables", "Evaluation History (X)", self.inspector_variable
+            monitor,
+            "Relative Variable Value",
+            "Relative Variable History (X)",
+            self.inspector_variable,
         )
 
         plot_var.setXLink(plot_obj)
@@ -392,7 +402,20 @@ class BadgerOptMonitor(QWidget):
         for i, name in enumerate(names):
             color = self.colors[i % len(self.colors)]
             # symbol = self.symbols[i % len(self.colors)]
-            _curve = plot_object.plot(pen=pg.mkPen(color, width=3), name=name)
+
+            # add a dot symbol to the plot to handle cases were there are many nans
+            dot_symbol = QtGui.QPainterPath()
+            size = 0.075  # size of the dot symbol
+            dot_symbol.addEllipse(QtCore.QRectF(-size / 2, -size / 2, size, size))
+
+            pen = pg.mkPen(color, width=3)
+            _curve = plot_object.plot(
+                pen=pen,
+                symbol=dot_symbol,
+                symbolPen=pen,
+                name=name,
+                symbolBrush=pen.color(),
+            )
             curves[name] = _curve
 
         return curves
@@ -453,7 +476,7 @@ class BadgerOptMonitor(QWidget):
     def open_extensions_palette(self):
         self.extensions_palette.show()
 
-    def extension_window_closed(self, child_window):
+    def extension_window_closed(self, child_window: AnalysisExtension):
         self.active_extensions.remove(child_window)
         self.extensions_palette.update_palette()
 
@@ -496,6 +519,9 @@ class BadgerOptMonitor(QWidget):
         if results is not None:
             self.routine.data = results
 
+        if not self.routine or not hasattr(self.routine, "sorted_data"):
+            # if no routine or sorted_data is available, return
+            return
         data_copy = self.routine.sorted_data
 
         # Get timestamps
@@ -734,8 +760,7 @@ class BadgerOptMonitor(QWidget):
         reply = QMessageBox.question(
             self,
             "Reset Environment",
-            f"Are you sure you want to reset the env vars "
-            f"back to {self.init_vars}?",
+            f"Are you sure you want to reset the env vars back to {self.init_vars}?",
             QMessageBox.Yes | QMessageBox.No,
             QMessageBox.No,
         )
@@ -744,7 +769,7 @@ class BadgerOptMonitor(QWidget):
 
         curr_vars = get_current_vars(self.routine)
 
-        self.routine.environment._set_variables(
+        self.routine.environment.set_variables(
             dict(zip(self.vocs.variable_names, self.init_vars))
         )
 
@@ -779,8 +804,7 @@ class BadgerOptMonitor(QWidget):
             QMessageBox.warning(
                 self,
                 "Jump to optimum",
-                "Jump to optimum is not supported for "
-                "multi-objective optimization yet",
+                "Jump to optimum is not supported for multi-objective optimization yet",
             )
 
     def jump_to_solution(self, idx):
@@ -805,11 +829,18 @@ class BadgerOptMonitor(QWidget):
             pos = idx = int(self.inspector_objective.value())
         variable_names = self.vocs.variable_names
         solution = df[variable_names].to_numpy()[idx]
+        curr_vars = get_current_vars(self.routine)
 
         reply = QMessageBox.question(
             self,
             "Apply Solution",
-            f"Are you sure you want to apply the selected solution at {solution} to env?",
+            "Are you sure you want to apply the selected solution:\n"
+            + "\n".join(
+                f"{variable_names[i]}: {round(curr_vars[i], 3)} -> {round(solution[i], 3)},"
+                for i in range(len(variable_names))
+            )
+            + "\nto "
+            + f"{self.routine.environment.name}?",
             QMessageBox.Yes | QMessageBox.No,
             QMessageBox.No,
         )
@@ -817,7 +848,13 @@ class BadgerOptMonitor(QWidget):
             return
 
         curr_vars = get_current_vars(self.routine)
-        self.routine.environment._set_variables(dict(zip(variable_names, solution)))
+        # Patch the hard limit to the environment
+        # TODO: this could lead to unexpected behavior
+        # since we patched the class variable directly
+        # there ought to be a better way to do this
+        if self.routine.vrange_hard_limit:
+            self.routine.environment.variables.update(self.routine.vrange_hard_limit)
+        self.routine.environment.set_variables(dict(zip(variable_names, solution)))
         # center around the inspector
         x_range = self.plot_var.getViewBox().viewRange()[0]
         delta = (x_range[1] - x_range[0]) / 2
@@ -825,7 +862,7 @@ class BadgerOptMonitor(QWidget):
 
         updated_vars = get_current_vars(self.routine)
         self.sig_status.emit(
-            f"Dial in solution: Env vars {curr_vars} -> {updated_vars}"
+            f"Dial in solution: {[f'{variable_names[i]}: {round(curr_vars[i], 4)} -> {round(updated_vars[i], 4)}' for i in range(len(variable_names))]}"
         )
         # QMessageBox.information(
         #     self, 'Set Environment', f'Env vars have been set to {solution}')
@@ -871,6 +908,15 @@ class BadgerOptMonitor(QWidget):
 
     def toggle_x_plot_y_axis_relative(self):
         self.x_plot_relative = self.check_relative.isChecked()
+
+        # Change axes labels depending on if relative is checked
+        if self.x_plot_relative:
+            self.plot_var.setLabel("left", "Relative Variable Value")
+            self.plot_var.setTitle("Relative Variable History (X)")
+        else:
+            self.plot_var.setLabel("left", "Variable Value")
+            self.plot_var.setTitle("Variable History (X)")
+
         self.update_curves()
 
     def on_mouse_click(self, event):
