@@ -16,12 +16,14 @@ class TestFactory:
             gen_config = get_generator_defaults(name)
             gen_class = get_generator(name)
 
-            if name in ["mobo", "cnsga", "mggpo"]:
-                gen_config["reference_point"] = {"y1": 10.0}
+            if name in ["mobo"]:
+                test_vocs = deepcopy(TEST_VOCS_BASE)
+                test_vocs.objectives = {"y1": "MINIMIZE", "y2": "MINIMIZE"}
+                gen_config["reference_point"] = {"y1": 10.0, "y2": 1.0}
                 json.dumps(gen_config)
 
-                gen_class(vocs=TEST_VOCS_BASE, **gen_config)
-            elif name in ["multi_fidelity"]:
+                gen_class(vocs=test_vocs, **gen_config)
+            elif name in ["multi_fidelity", "extremum_seeking", "neldermead", "rcds"]:
                 test_vocs = deepcopy(TEST_VOCS_BASE)
                 test_vocs.constraints = {}
                 json.dumps(gen_config)
